
/* darkstat: a network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* Everything here is covered by the GPL (see the 'COPYING' file) */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h> /* for ntohs() and friends */
#include <net/if.h>
#include <arpa/inet.h>
#include <pcap.h>

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#if defined(sun) && (defined(__svr4__) || defined(__SVR4)) 
#include <sys/sockio.h>
#endif
#include <locale.h>

#include "darkstat.h"
#include "host_db.h"
#include "port_db.h"
#include "proto.h"
#include "bignum.h"
#include "dns.h"
#include "www.h"
#include "graph.h"

#define CAP_PACKETS 100
#define EXIT_SUCCESS 0
#define EXIT_FAIL 1
#define SAVE_TIME 60*5 /* secs inbetween DB dumps */

i64	num_packets, total_data;
dword	local_ip = 0, lan_ip = 0xFFFFFFFF, lan_mask = 0, local_host_num;
int	verbose = 0, is_ether = 1, webport = DEFAULT_WEBPORT,
	shutting_down = 0, linktype = 0;
char	*errdb_file = NULL, *db_file = NULL, *daylog_file = NULL;
time_t	t_start, t_already = 0, t_lastsave = 0;
pcap_t	*pcap = NULL;



char *build_str(char *format, ...)
{
	va_list va;
	char buf[1024], *s;
	int count;

	va_start(va, format);
	count = vsnprintf(buf, 1024, format, va);

	if (count == -1) freakout("buffer overflow in build_str()");
	va_end(va);

	s = (char*)malloc(strlen(buf) + 1);
	if (!s) freakout("ran out of memory in build_str()");
	strcpy(s, buf);

	return s;
}



void freakout(const char *err)
{
	fprintf(stderr, "\n>>> %s <<<\n\n"
		"Something has gone terribly wrong.\n"
		"I'm going to try to dump db to minimize data loss.\n",
		err);

	save_db(errdb_file);
	fprintf(stderr, "Dying now...\n");
	if (pcap) pcap_close(pcap);
	exit(EXIT_FAIL);
}



void print_addr(const dword ip)
{
	printf("%d.%d.%d.%d",
		(ip >> 24) & 255,
		(ip >> 16) & 255,
		(ip >> 8) & 255,
		(ip & 255) );
}



/* Packet decoding - hints from ACSys and tcpdump */
#define ETH_PKT_TYPE_IP 0x0800
inline word eth_pkt_type(const byte *pkt)
{
	return ntohs( ((word*)pkt)[6] );
}

inline byte *eth_pkt_strip(byte *pkt)
{
	return pkt+14;
}


#define IP_PROTO_ICMP 1 /* Internet Control Message Protocol */
#define IP_PROTO_TCP 6  /* Transmission Control Protocol */
#define IP_PROTO_UDP 17 /* User Datagram Protocol */
inline word ip_pkt_len(const byte *pkt)
{
	return ntohs( ((word*)pkt)[1] );
}

inline u_char ip_pkt_proto(const byte *pkt)
{
	return pkt[9];
}

inline dword ip_pkt_srcip(const byte *pkt)
{
	return ntohl( ((dword*)(pkt+12))[0] );
}

inline dword ip_pkt_destip(const byte *pkt)
{
	return ntohl( ((dword*)(pkt+16))[0] );
}

inline u_char *ip_pkt_strip(u_char *pkt)
{
	return pkt+20;
}

inline word tcp_pkt_srcport(const byte *pkt)
{
	return ntohs( ((word*)pkt)[0] );
}

inline word tcp_pkt_destport(const byte *pkt)
{
	return ntohs( ((word*)pkt)[1] );
}

inline word tcp_pkt_seq(const byte *pkt)
{
	return ntohl( ((dword *)pkt)[1] );
}

inline word tcp_pkt_ack(const byte *pkt)
{
	return ntohl( ((dword *)pkt)[2] );
}

inline u_char tcp_flags(const byte *pkt)
{
	return pkt[13] & 0x3F;
}

#define TCP_URG (1<<5)
#define TCP_ACK (1<<4)
#define TCP_PSH (1<<3)
#define TCP_RST (1<<2)
#define TCP_SYN (1<<1)
#define TCP_FIN (1<<0)



/* this is the heart of the beast */
void handle_pkt(u_char *user, const struct pcap_pkthdr *pheader,
	u_char *pdata)
{
	u_char *ptr;
	word pkt_type;
	dword pkt_len;
	u_char pkt_proto;
	dword pkt_srcip, pkt_destip;
	port_record *p;

	if (shutting_down) return; /* blarg */

	/* update total packet and data counters */
	add_64bit(num_packets, 1);
	add_64bit(total_data, (dword)(pheader->len));

	/* in the debug compile _just in case_ */
	assert(strcmp(user, "abc") == 0);

	if (!is_ether)
	{
		ptr = pdata;
		while (!ptr[0]) ptr++;

		ptr -= 12;
	}

	if (linktype == DLT_RAW)
	{
		/* raw packets have no header to skip! */
		ptr = pdata;
		pkt_type = ETH_PKT_TYPE_IP;
	}
	else
	{
		if (linktype == DLT_FDDI)
		{
			ptr = pdata + 13; /* skip FDDI header */
			ptr += 6; /* some other garbage */
			ptr -= 12; /* now it matches up with ether */
		}
		else ptr = pdata;

		pkt_type = eth_pkt_type(ptr);

		if (pkt_type != ETH_PKT_TYPE_IP) {
			if (verbose)
			{
				putchar('(');
				switch (pkt_type)
				{
				case 0x0806: printf("ARP"); break;
				case 0x809b: printf("AppleTalk"); break;
				case 0x8035: printf("RARP"); break;
				case 0x814c: printf("SNMP"); break;
				case 0x86dd: printf("IPv6"); break;
				case 0x880b: printf("PPP"); break;
				default: printf("unknown");
				}
				printf(" packet -- ignored)\n");
				fflush(stdout);
			}
			return;
		}

		ptr = eth_pkt_strip(ptr);
	}

	if (verbose) printf("(IP) ");

	pkt_proto = ip_pkt_proto(ptr);
	pkt_srcip = ip_pkt_srcip(ptr);
	pkt_destip = ip_pkt_destip(ptr);
	pkt_len = (dword)(ip_pkt_len(ptr));

	/* de-mangle Linux 2.4.x NAT */
	if ((pkt_srcip & lan_mask) == lan_ip) pkt_srcip = local_ip;
	if ((pkt_destip & lan_mask) == lan_ip) pkt_destip = local_ip;

	if (verbose)
	{
		printf(proto_name_short(pkt_proto));
		putchar(' ');
		print_addr(pkt_srcip);
		printf("->");
		print_addr(pkt_destip);
	}

	/* start meddling with db */
	/* pthread_mutex_lock(&db_mutex); */

	/* accounting for protocols */
	if (pkt_srcip == local_ip)
		proto_transfer_out(pkt_proto, pkt_len); else
	if (pkt_destip == local_ip)
		proto_transfer_in(pkt_proto, pkt_len);

	/* accounting for hosts */
	host_transfer_out(pkt_srcip, pkt_len);
	host_transfer_in(pkt_destip, pkt_len);

	/* accounting for ports */
	if (pkt_proto == IP_PROTO_TCP)
	{
		u_char *tcp_ptr = ip_pkt_strip(ptr);
		word	srcport = tcp_pkt_srcport(tcp_ptr),
			destport = tcp_pkt_destport(tcp_ptr);
		u_char flags = tcp_flags(tcp_ptr);

		if (pkt_destip == local_ip)
		{
			if ((flags & TCP_SYN) && !(flags ^ TCP_SYN))
				p = port_from_num(destport);
			else
				p = port_find(destport);
			if (p) port_update_in(p, pkt_len);
		}
	       	else if (pkt_srcip == local_ip)
		{
			p = port_find(srcport);
			if (p) port_update_out(p, pkt_len);
		}	

		if (verbose) {
			printf(" %d:%d (", srcport,destport);
		
			if (flags & TCP_URG) printf("URG ");
			if (flags & TCP_ACK) printf("ACK ");
			if (flags & TCP_PSH) printf("PSH ");
			if (flags & TCP_RST) printf("RST ");
			if (flags & TCP_SYN) printf("SYN ");
			if (flags & TCP_FIN) printf("FIN ");
			if (flags) putchar(8);
			printf(") ");

			printf("seq=%u", tcp_pkt_seq(tcp_ptr));
			if (flags & TCP_ACK)
				printf(", ack=%u", tcp_pkt_ack(tcp_ptr));
		}
	}

	/* stop meddling with db */
	/* pthread_mutex_unlock(&db_mutex); */

	/* update graph */
	pthread_mutex_lock(&graph_mutex);
	graph_rotate(pheader->ts.tv_sec);
	if (pkt_srcip == local_ip) graph_add_out(pkt_len); else
	if (pkt_destip == local_ip) graph_add_in(pkt_len);
	pthread_mutex_unlock(&graph_mutex);

	if (verbose)
	{
		printf(" size:%d\n", pkt_len);
		fflush(stdout);
	}
}



void init_db(void)
{
	host_db_init();
	port_db_init();
	proto_db_init();
}

void free_db(void)
{
	host_db_free();
	port_db_free();
}

void save_db(const char *filename)
{
	time_t uptime = t_already + (time(NULL) - t_start);
	FILE *fp = fopen(filename, "w");
	if (!fp) {
		fprintf(stderr,"save_db(): Can't open file %s to dump db.\n",
			filename);
		exit(EXIT_FAIL);
	}

#ifdef HAVE_QUAD
	fprintf(fp,"%u\n%u\n%u\n%u\n%u\n", (dword)uptime,
		HI(num_packets), LO(num_packets),
		HI(total_data), LO(total_data));
#else
	fprintf(fp,"%u\n%u\n%u\n%u\n%u\n", (dword)uptime,
		num_packets.hi, num_packets.lo,
		total_data.hi, total_data.lo);
#endif

	pthread_mutex_lock(&db_mutex);
	host_db_dump(fp);
	proto_db_dump(fp);
	port_db_dump(fp);
	pthread_mutex_unlock(&db_mutex);

	graph_save(fp);

	fclose(fp);
}

/* I loathe this function.  Eventually the database will be replaced by
 * a tidy[er] binary format and this function will be cleaned right up.
 */
void load_db(char *filename)
{
	FILE *fp = fopen(filename, "r");
	char buf[1024], buf2[1024];
	int i,j,k,l, hi, lo;
	
	if (!fp) {
		printf("Can't load db from %s, starting from scratch.\n",
			filename);
		return;
	}

#define getline if (!fgets(buf,1024,fp)) {\
	fprintf(stderr, "load_db() failed: invalid data "\
		"in %s\n", filename); exit(101); }
	
#define atodw(buf) (dword)strtoul(buf,NULL,10)
	
#define parse(delimiter) { l += k+1;		\
		k = 0;				\
		while (buf[l+k] != delimiter)	\
		{				\
			buf2[k] = buf[l+k];	\
			k++;			\
		}				\
		buf2[k] = 0; }

	/* time */
	getline; t_already = atodw(buf);

	/* totals */
	getline; hi=atodw(buf);
	getline; lo=atodw(buf);
	SET64(num_packets, hi, lo);

	getline; hi=atodw(buf);
	getline; lo=atodw(buf);
	SET64(total_data, hi, lo);

	/* hosts */
	getline;
	j = atodw(buf);
	for (i=0; i<j; i++)
	{
		host_record *h;

		getline;
		
		/* field 1: ip */
		l=-1; k=0;
		parse('|');

		h = host_make(atodw(buf2));

		/* field 2: domain name */
		parse('|');
		if (strlen(buf2) == 0)
			h->hostname = NULL;
		else h->hostname = strdup(buf2);

		/* field 3: data in pow32 */
		parse(':');
		hi = atodw(buf2);	

		/* field 4: data in */
		parse('|');
		lo = atodw(buf2);	
		SET64(h->data_in, hi, lo);

		/* field 5: data out pow32 */
		parse(':');
		hi = atodw(buf2);	

		/* field 6: data out */
		parse('\n');
		lo = atodw(buf2);	
		SET64(h->data_out, hi, lo);

		if (!db_add(&host_db, h)) freakout("Error loading hosts");
	}	
	printf("Loaded hosts.\n"); fflush(stdout);

	/* protocols */
	for (i=0; i<256; i++)
	{
		getline;
		l=-1; k=0;
		parse(':'); hi = atodw(buf2);
		parse('|'); lo = atodw(buf2);
		SET64(proto_in[i], hi, lo);
		parse(':'); hi = atodw(buf2);
		parse('\n'); lo = atodw(buf2);
		SET64(proto_out[i], hi, lo);
	}

	printf("Loaded protocols.\n"); fflush(stdout);

	/* tcp ports */
	getline;
	j = atodw(buf);
	for (i=0; i<j; i++)
	{
		port_record *p;
		
		getline;
		
		/* field 1: port number */
		l=-1; k=0;
		parse('|');

		p = port_make(atodw(buf2));
	
		/* field 2: data in pow32 */
		parse(':');
		hi = atodw(buf2);	

		/* field 3: data in */
		parse('|');
		lo = atodw(buf2);
		SET64(p->data_in, hi, lo);

		/* field 4: data out pow32 */
		parse(':');
		hi = atodw(buf2);	

		/* field 5: data out */
		parse('\n');
		lo = atodw(buf2);	
		SET64(p->data_out, hi, lo);

		if (!db_add(&port_db, p)) freakout("Error loading ports");
	}

	printf("Loaded ports.\n"); fflush(stdout);

#define FREAK(x) { fprintf(stderr,x "\n"); fflush(stderr); exit(101); }

	/* graphs */
#define LOAD_GRAPH(GRAPH_SIZE,UNITS,pos,counter,graph_in,graph_out) { \
	getline;		\
	l=-1; k=0;		\
	parse('/');		\
	pos = atodw(buf2);	\
	parse(' ');		\
	j = atodw(buf2);	\
	parse('\n');		\
	counter = atodw(buf2);	\
	if (j != GRAPH_SIZE) FREAK("Wrong number of " UNITS \
		" in saved graph data!"); \
	for (i=0; i<GRAPH_SIZE; i++) \
	{			\
		getline;	\
				\
		/* in32:in out32:out */ \
		l=-1; k=0;	\
		parse(':');	\
		hi = atodw(buf2); \
		parse(' ');	\
		lo = atodw(buf2); \
		SET64(graph_in[i], hi, lo); \
				\
		parse(':');	\
		hi = atodw(buf2); \
		parse('\n');	\
		lo = atodw(buf2); \
		SET64(graph_out[i], hi, lo); \
	} }

	LOAD_GRAPH(GRAPH_SECS,"seconds",pos_sec,g_secs,
			graph_sec_in,graph_sec_out);
	LOAD_GRAPH(GRAPH_MINS,"minutes",pos_min,g_mins,
			graph_min_in,graph_min_out);
	LOAD_GRAPH(GRAPH_HRS,"hours",pos_hr,g_hrs,
			graph_hr_in,graph_hr_out);
	LOAD_GRAPH(GRAPH_DAYS,"days",pos_day,i,
			graph_day_in,graph_day_out);

	printf("Loaded graphs.\n"); fflush(stdout);

#undef LOAD_GRAPH
#undef FREAK
#undef getline
#undef atodw
#undef parse
	
	fclose(fp);
}



RETSIGTYPE term_signal(int signum)
{
	if (shutting_down)
	{
		printf("Caught a redundant SIGTERM. Ignored.\n");
		return;
	}
	shutting_down = 1;
	printf("Caught SIGTERM, syncing threads...\n");

	pthread_join(www_thread, NULL);
	pthread_join(dns_thread, NULL);

	printf("Synced!  Dumping DB...\n");

	save_db(db_file);
	printf("Done!\n");
	
	free(db_file);
	free(errdb_file);
	free(daylog_file);

	pthread_mutex_destroy(&db_mutex);
	pthread_mutex_destroy(&graph_mutex);

	free_db();

	/* pcap_close(pcap); */ /* causes long delay under FreeBSD */

	exit(EXIT_SUCCESS);
}



dword get_local_ip(char *inter)
{
	int tmp = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
	struct ifreq ifr;
	struct sockaddr sa;

	strcpy(ifr.ifr_name, inter);
	ifr.ifr_addr.sa_family = AF_INET;
	if (ioctl(tmp, SIOCGIFADDR, &ifr) != 0)
	{
		fprintf(stderr, "Can't get own IP address on interface %s.\n",
			inter);
		exit(101);
	}
	close(tmp);
	sa = ifr.ifr_addr;

/* linux/socket.h:
 * struct sockaddr {
 *	sa_family_t     sa_family;      * address family, AF_xxx       
 *	char            sa_data[14];    * 14 bytes of protocol address 
 */

/* linux/in.h:
 * struct sockaddr_in {
 * sa_family_t           sin_family;     * Address family
 *	unsigned short int    sin_port;       * Port number
 *	struct in_addr        sin_addr;       * Internet address
 * * Pad to size of `struct sockaddr'.
 *
 * * Internet address.
 * struct in_addr {
 *	__u32   s_addr;
 */

	return ntohl( ((struct sockaddr_in*)&sa)->sin_addr.s_addr );
}



int main(int argc, char **argv)
{
	char *netdev, err[PCAP_ERRBUF_SIZE], *fprefix = NULL;
	struct pcap_stat ps;
	int i;

#ifdef HAVE_QUAD
	/* thousands separators */
	setlocale(LC_ALL, "en_US");
#endif

	printf(PACKAGE " v" VERSION " using libpcap v%d.%d\n",
		PCAP_VERSION_MAJOR, PCAP_VERSION_MINOR);

	netdev = pcap_lookupdev(err);

	/* handle commandline arguments */
	for (i=1; i<argc; i++)
	{
		if ((strcmp(argv[i], "-h") == 0) ||
			(strcmp(argv[i], "--help") == 0))
		{
			printf(
"usage: %s -i <interface> [-p <number>] [-d <path>] [-l <ip>/<mask>] \n"
"\t\t[-v] [-h] [-V]\n\n"
"  -i\tspecify which interface to listen on (default: %s)\n"
"  -p\tweb interface port (default: %d)\n"
"  -d\tdatabase storage directory (default: working directory)\n"
"  -v\tverbose mode - packet dumps and more\n"
"  -h\thelp (what you're looking at now)\n"
"  -n\tdon't resolve IPs to hostnames\n"
"  -l\tlocal network range for correct SNAT accounting (Linux 2.4.x)\n"
"  -V\tversion information\n\n",
				argv[0], netdev, webport);

			return EXIT_SUCCESS;
		}
		else if (strcmp(argv[i], "-v") == 0) verbose = 1;
		else if (strcmp(argv[i], "-n") == 0) want_resolve = OFF;
		else if (strcmp(argv[i], "-p") == 0)
		{
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Port not specified!\n\n");
				return EXIT_FAIL;
			}
			webport = atoi(argv[i]);
		}
		else if (strcmp(argv[i], "-i") == 0)
		{
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Interface not specified!\n\n");
				return EXIT_FAIL;
			}
			netdev = argv[i];
		}
		else if (strcmp(argv[i], "-d") == 0)
		{
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("Directory not specified!\n\n");
				return EXIT_FAIL;
			}
			fprefix = build_str("%s%s", argv[i],
				argv[i][strlen(argv[i])-1] == '/'?"":"/");
		}
		else if ((strcmp(argv[i], "-V") == 0) ||
			(strcmp(argv[i], "--version") == 0))
		{
			printf(COPYRIGHT "\n" URL "\n\n"
"\tThis program is free software; you can redistribute it and/or modify\n"
"\tit under the terms of the GNU General Public License as published by\n"
"\tthe Free Software Foundation; either version 2 of the License, or\n"
"\t(at your option) any later version.\n\n"
"\tThis program is distributed in the hope that it will be useful,\n"
"\tbut WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"\tMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"\tGNU General Public License for more details.\n\n"
"\tYou should have received a copy of the GNU General Public License\n"
"\talong with this program. If not, write to the Free Software\n"
"\tFoundation, 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"
			);
			return EXIT_SUCCESS;
		}
		else if (strcmp(argv[i], "-l") == 0)
		{
			int i1,i2,i3,i4, m1,m2,m3,m4;
			
			i++;
			if ((argc < i+1) || (argv[i][0] == '-'))
			{
				printf("LAN space not specified!\n\n");
				return EXIT_FAIL;
			}
			sscanf(argv[i], "%d.%d.%d.%d/%d.%d.%d.%d",
				&i1, &i2, &i3, &i4,
				&m1, &m2, &m3, &m4);

			lan_ip = ip_from_quad(i1,i2,i3,i4);
			lan_mask = ip_from_quad(m1,m2,m3,m4);

			printf("Local LAN is %d.%d.%d.%d/%d.%d.%d.%d\n",
				i1,i2,i3,i4, m1,m2,m3,m4);
		}
		else {
			printf("Invalid argument: %s\n", argv[i]);
			return EXIT_FAIL;
		}
	}

	/* database files, accounting for the specified storage directory */
	if (fprefix == NULL) fprefix="";
	db_file = build_str("%s" DB_FILE_NAME, fprefix);
	errdb_file = build_str("%s" ERRDB_FILE_NAME, fprefix);
	daylog_file = build_str("%s" DAYLOG_FILE_NAME, fprefix);

	if (strcmp(netdev,"lo") == 0) is_ether = 0;

	local_ip = get_local_ip(netdev);
	printf("Sniffing on device %s, local IP is ", netdev);
	print_addr(local_ip);
	putchar('\n');

	init_db();
	init_graph();
	load_db(db_file);

	if (host_db.used == 0)
	{
		db_add(&host_db, host_make(local_ip));
		local_host_num = 0;
	} else {
		dword i;

		local_host_num = -1;
		for (i=0; i<host_db.used; i++)
			if ( ((host_record*)
				db_get(host_db,i))->ip_addr == local_ip)
					local_host_num = i;

		if (local_host_num == -1)
		{
			local_host_num = host_db.used;
			db_add(&host_db, host_make(local_ip));
		}
	}

	/* the only way out */
	signal(SIGTERM, term_signal);
	signal(SIGINT, term_signal);

	/* fire up threads */
	printf("Firing up threads...\n"); fflush(stdout);

	if (pthread_mutex_init(&graph_mutex, NULL) == -1)
	{
		fprintf(stderr,"Could not create graph mutex.\n");
		return EXIT_FAIL;
	}
	if (pthread_mutex_init(&db_mutex, NULL) == -1)
	{
		fprintf(stderr,"Could not create db mutex.\n");
		return EXIT_FAIL;
	}
	if (pthread_create(&dns_thread, NULL, (void*)&dns_main, NULL) == -1)
	{
		fprintf(stderr,"Could not create DNS thread.\n");
		return EXIT_FAIL;
	}
	if (pthread_create(&www_thread, NULL, (void*)&www_main, NULL) == -1)
	{
		fprintf(stderr,"Could not create WWW thread.\n");
		return EXIT_FAIL;
	}

	pcap = pcap_open_live(netdev, 1024, 1, 1000, err);
	if (!pcap)
	{
		fprintf(stderr, "Can't pcap_open_live()! "
			"Are you not running as root?\n");
		return EXIT_FAIL;
	}
	
	t_lastsave = t_start = time(NULL);
	linktype = pcap_datalink(pcap);

	if (!pcap) {
		fprintf(stderr, "pcap_open_live() failed: %s\n", err);
		return EXIT_FAIL;
	}

	printf( "Capturing traffic...\n"
		"Point your browser to http://localhost:%d/ to see the stats."
		"\n\n", webport);

	while (!shutting_down)
	{
		/* capture some packets */
		pcap_loop(pcap, CAP_PACKETS,
			(pcap_handler)handle_pkt, "abc");

		/* print out cap statistics */	
		pcap_stats(pcap, &ps);
		if (verbose)
			printf("Packets: received %d, dropped %d\n",
				ps.ps_recv, ps.ps_drop);

		/* commit db to disk */
		if (time(NULL) - t_lastsave > SAVE_TIME)
		{
			t_lastsave = time(NULL);
			save_db(db_file);
		}
	}

	printf("Main thread is out of capture loop.\n");
	while (shutting_down) usleep(1000*MSEC);

	assert(0 && "Execution should never reach this point!");
	return EXIT_FAIL;
}

