
/* Network traffic analyzer
 * (c) 2001-2002, Emil Mikulic.
 */

/* Host database */
#include "host_db.h"



/* Global database */
db host_db;



void host_db_init(void)
{
	db_init(&host_db);
}

void host_db_free(void)
{
	unsigned int i;

	for (i=0; i<host_db.used; i++)
		free(host_db.records[i]);

	db_free(&host_db);
}



/* host_record needs to be deallocated when it's no longer needed */
inline host_record *host_make(const dword ip_addr)
{
	host_record *h = (host_record*)malloc(sizeof(host_record));

	h->ip_addr = ip_addr;
	h->hostname = NULL;
	SET64(h->data_in, 0, 0);
	SET64(h->data_out, 0, 0);

	return h;
}



/* update transfer amount */
inline void host_update_in(host_record *h, const dword in)
{
	add_64bit(h->data_in, in);
}

inline void host_update_out(host_record *h, const dword out)
{
	add_64bit(h->data_out, out);
}



/* add or update a host, depending on whether it exists in the db or not */
inline host_record *host_from_ip(const dword ip)
{
	int i;
	host_record *h = NULL;
	extern dword local_ip;

	assert(host_db.used > 0);

	if (ip == local_ip) {
		h = (host_record*)db_get(host_db, local_host_num);
		assert(h->ip_addr == ip);

		return h;
	}

	/* search backwards - more likely to find a more recent host quickly */
	for (i=host_db.used-1; i>=0; i--)
	{
		host_record *tmp = (host_record*)db_get(host_db, i);
		if (tmp->ip_addr == ip) h = tmp;
	}

	if (!h)
	{
		h = host_make(ip);
		if (!db_add(&host_db, h)) freakout("Couldn't add to host_db in host_from_ip()");
	}

	assert(h->ip_addr == ip);
	return h;
}



inline void host_transfer_in(const dword ip, const dword in)
{
	host_update_in(host_from_ip(ip), in);
}

inline void host_transfer_out(const dword ip, const dword out)
{
	host_update_out(host_from_ip(ip), out);
}



inline dword ip_from_quad(const int a, const int b, const int c, const int d)
{
	return (
			((a & 255) << 24) |
			((b & 255) << 16) |
			((c & 255) << 8) |
			(d & 255)
		);
}



void host_db_dump(FILE *fp)
{
	int i;

	fprintf(fp, "%u\n", host_db.used);
	for (i=0; i<host_db.used; i++)
	{
		host_record *h = (host_record*)db_get(host_db, i);
		
		fprintf(fp, "%u|%s|%u:%u|%u:%u\n",
			h->ip_addr,
			(h->hostname)?h->hostname:"",
			HI(h->data_in),
			LO(h->data_in),
			HI(h->data_out),
			LO(h->data_out));
	}
}



void host_qs_swap(host_record **list, i64 *amount, int a, int b)
{
	i64 t64;
	host_record *thr;

	/*
	assert(a >= 0);
	assert(b >= 0);
	assert(a < host_db.used);
	assert(b < host_db.used);
	 */

	t64 = amount[a];
	amount[a] = amount[b];
	amount[b] = t64;

	thr = list[a];
	list[a] = list[b];
	list[b] = thr;
}



void host_quicksort(host_record **list, i64 *amount, int left, int right)
{
	int i, j;
	i64 v;

	if(right > left) {
		v = amount[right];
		i = left - 1;
		j = right;
		for(;;) {
			do {
				i++;
				if (i >= j) break;
			} while (smaller_64(amount[i], v));

			do {
				j--;
				if (i >= j) break;
			} while (bigger_64(amount[j], v));

			if (i >= j) break;
			host_qs_swap(list, amount, i, j);
		}
		host_qs_swap(list, amount, i, right);
		host_quicksort(list, amount, left, i - 1);
		host_quicksort(list, amount, i + 1, right);
	}
}



/* note: it'd be a good idea to lock the host_db mutex before doing this */
inline host_record **host_db_sort(sort_type st)
{
	host_record **list;
	i64 *amount;
	int i;

	list = (host_record**) malloc(host_db.used * sizeof(host_record*));
	amount = (i64*) malloc(host_db.used * sizeof(i64));

	for (i=0; i<host_db.used; i++)
	{
		/* initialize host record */
		list[i] = (host_record*)db_get(host_db, i);

		/* initialize amount */
		switch(st)
		{
			case TOTAL:
#ifdef HAVE_QUAD
				amount[i] = list[i]->data_in + list[i]->data_out;
#else
				amount[i] = list[i]->data_in;			/* total = in */
				add_64bit(amount[i], list[i]->data_out.lo);	/* plus out.lo */
				amount[i].hi += list[i]->data_out.hi;		/* plus out.hi */
#endif
				break;
			case IN:
				amount[i] = list[i]->data_in;
				break;
			case OUT:
				amount[i] = list[i]->data_out;
				break;
			default:
				freakout("invalid sort_type passed to host_db_sort()");
		}
	}

	host_quicksort(list, amount, 0, host_db.used-1);

	free(amount);
	return list;
}

